﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=7.01
@EndOfDesignText@
'Custom View class 
'JustImgSlider - simple manual image slider, v.0.3
'(c) 26.05.2017 Peacemaker
' Additions by Andrew (Digitwell) to support centering of images, setting the background colour and setting the color of the dots
'#Event: ExampleEvent (Value As Int)
'#DesignerProperty: Key: BooleanExample, DisplayName: Boolean Example, FieldType: Boolean, DefaultValue: True, Description: Example of a boolean property.
'#DesignerProperty: Key: IntExample, DisplayName: Int Example, FieldType: Int, DefaultValue: 10, MinRange: 0, MaxRange: 100, Description: Note that MinRange and MaxRange are optional.
'#DesignerProperty: Key: StringWithListExample, DisplayName: String With List, FieldType: String, DefaultValue: Sunday, List: Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday
'#DesignerProperty: Key: StringExample, DisplayName: String Example, FieldType: String, DefaultValue: Text
'#DesignerProperty: Key: ColorExample, DisplayName: Color Example, FieldType: Color, DefaultValue: 0xFFCFDCDC, Description: You can use the built-in color picker to find the color values.
'#DesignerProperty: Key: DefaultColorExample, DisplayName: Default Color Example, FieldType: Color, DefaultValue: Null, Description: Setting the default value to Null means that a nullable field will be displayed.
#DesignerProperty:  Key: DotsColor, DisplayName: Dots Colour, FieldType: Color, DefaultValue: 0xFFFFFFFF, Description: Set the colour of the image dots.
#DesignerProperty:  Key: PanelBackground, DisplayName: Panel Background, FieldType: Color, DefaultValue: 0xFFFFFFFF, Description: Set the colour of the panel background when centering image.

Sub Class_Globals
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private mBase As Panel
	Private Const DefaultColorConstant As Int = -984833 'ignore
	Private lstSlides As List
	Private lblStatus As Label
	Type Slide (Name As String, URI As String, Pnl As Panel)
	Private CurrentSlideID, NextID As Int
	Private PrevX As Int, LeftDirection As Boolean
	Private DotColor As Long
	Private PnlBckGrnd As Long
End Sub

Public Sub Initialize (Callback As Object, EventName As String)
	mEventName = EventName
	mCallBack = Callback
	lstSlides.Initialize
End Sub

Public Sub DesignerCreateView (Base As Panel, Lbl As Label, Props As Map)
	mBase = Base
	lstSlides.Initialize
	lblStatus.Initialize("")
	lblStatus.Text = "No images"
	#IF B4A
	lblStatus.Gravity = Bit.Or(Gravity.CENTER_HORIZONTAL, Gravity.CENTER_VERTICAL)
	#Else IF B4i
	lblStatus.TextAlignment = lblStatus.ALIGNMENT_CENTER
	#End If
	
	If lblStatus.IsInitialized Then
		#if B4A
		lblStatus.RemoveView
		#else if B4i
		lblStatus.RemoveViewFromParent
		#End If
		mBase.AddView(lblStatus, (mBase.Width - mBase.Width)/2, mBase.Height - 20dip, mBase.Width, 20dip)
	End If
	
	DotColor = Props.Get("DotsColor")
	PnlBckGrnd = Props.Get("PanelBackground")
	
	#if b4i
	lblStatus.Font = Font.CreateFontAwesome(17)
	#end if
	lblStatus.TextColor = DotColor
 
End Sub

Private Sub Base_Resize (Width As Double, Height As Double)
	mBase.Width = Width
	mBase.Height = Height
	lblStatus.Left = (mBase.Width - mBase.Width)/2
	lblStatus.Top = mBase.Height - 20dip
End Sub

Public Sub GetBase As Panel
	Return mBase
End Sub

#Region Code-manipulations

Public Sub getLeft As Int
	Return mBase.Left
End Sub
Public Sub setLeft(Left As Int)
	mBase.Left = Left
End Sub
Public Sub getTop As Int
	Return mBase.Top
End Sub
Public Sub setTop(Top As Int)
	mBase.Left = Top
End Sub
Public Sub getWidth As Int
	Return mBase.Width
End Sub
Public Sub setWidth(Width As Int)
	mBase.Width = Width
End Sub
Public Sub getHeight As Int
	Return mBase.Height
End Sub
Public Sub setHeight(Height As Int)
	mBase.Height = Height
End Sub
#End Region

'add an image URL
Public Sub AddSlide (Name As String, URL As String,center As Boolean)
	Dim a As Slide
	a.Initialize
	a.Name = Name
	a.URI = URL
	
	
	Dim p As Panel
	p.Initialize("pnlSlide")
	mBase.AddView(p, 0, 0, mBase.Width, mBase.Height)
	p.SendToBack
	p.Color = PnlBckGrnd
	a.pnl = p
	a.pnl.Tag = lstSlides.Size	'record id for reference
	lstSlides.Add(a)
	DownloadImage(URL, p,center)
	Update_Status
End Sub


Public Sub AddSlidewithCredentials (Name As String, URL As String, username As String, password As String,center As Boolean)
	Dim a As Slide
	a.Initialize
	a.Name = Name
	a.URI = URL
	
	Dim p As Panel
	p.Initialize("pnlSlide")
	mBase.AddView(p, 0, 0, mBase.Width, mBase.Height)
	p.SendToBack
	p.Color = PnlBckGrnd
	a.pnl = p
	a.pnl.Tag = lstSlides.Size	'record id for reference
	lstSlides.Add(a)
	DownloadImagewithCredentials(URL, p,username,password,center)
	Update_Status
End Sub


'add a local image file
Public Sub AddImageFile (Name As String, Folder As String, FileName As String,center As Boolean)
	Dim a As Slide
	a.Initialize
	a.Name = Name
	a.URI = File.Combine(Folder, FileName)
	
	Dim p As Panel
	p.Initialize("pnlSlide")
	Log("width = " & mBase.width & "  Height="&mBase.Height)
	mBase.AddView(p, 0, 0, mBase.Width, mBase.Height)
	p.SendToBack
	p.Color = PnlBckGrnd
	a.pnl = p
	a.pnl.Tag = lstSlides.Size	'record id for reference
	lstSlides.Add(a)
	If center Then
		FitCenterBitmapPanel(p,LoadBitmap(Folder,FileName),PnlBckGrnd)
	Else
		#if B4A
		p.SetBackgroundImage(LoadBitmap(Folder, FileName))
		#else if B4i
		SetBackgroundImage(p,LoadBitmap(Folder,FileName))
		#End If
	End If
	Update_Status
End Sub

'remove all slides
Public Sub Clear
	For i = 0 To lstSlides.Size - 1
		Dim a As Slide
		a = lstSlides.Get(i)
		#if B4a
		a.pnl.SetBackgroundImage(Null)
		a.pnl.RemoveView
		#else if B4i
		SetBackgroundImage(a.pnl,Null)
		a.pnl.RemoveViewFromParent
		#End If
	Next
	lstSlides.Clear
	Update_Status
End Sub

Sub DownloadImage(Link As String, pnl As Panel,center As Boolean)
	Dim j As HttpJob
	j.Initialize("", Me)
	j.Download(Link)
	Wait For (j) JobDone(j As HttpJob)
	If j.Success Then
		If center Then
			FitCenterBitmapPanel(pnl,j.GetBitmap,PnlBckGrnd)
		Else
			#if b4a
			pnl.SetBackgroundImage(j.GetBitmap)
			#else if b4i
			SetBackgroundImage(pnl,j.GetBitmap)
			#end if
		End If
	End If
	j.Release
End Sub



Sub DownloadImagewithCredentials(Link As String, pnl As Panel,username As String ,password As String,center As Boolean)
	Dim j As HttpJob
	j.Initialize("", Me)
	j.Username = username
	j.Password = password
	j.Download(Link)
	
	Wait For (j) JobDone(j As HttpJob)
	If j.Success Then
		If center Then
			FitCenterBitmapPanel(pnl,j.GetBitmap,PnlBckGrnd)
		Else
			#if b4a
			pnl.SetBackgroundImage(j.GetBitmap)
			#else if b4i
			SetBackgroundImage(pnl,j.GetBitmap)
			#end if
		End If
	End If
	j.Release
End Sub

Sub pnlSlide_Touch (Action As Int, X As Float, Y As Float)
	If lstSlides.Size <= 1 Then
		Return
	End If
	Dim pnl As Panel = Sender
	
	If Action = pnl.ACTION_DOWN Then
		PrevX = X
	Else If Action = pnl.ACTION_MOVE Then
		If X <= PrevX Then
			LeftDirection = True
		Else
			LeftDirection = False
		End If
		If pnl.Left < 0 Then	'next slide will be .....
			NextID = CurrentSlideID + 1
		Else
			NextID = CurrentSlideID - 1
		End If
		Check_Next
		Hide_others
		
		If Abs(X - PrevX) > 2%x Then
			If LeftDirection Then
				Dim delta As Int = pnl.Left - Abs(PrevX - X)/2
			Else
				Dim delta As Int = pnl.Left + Abs(PrevX - X)/2
			End If
			#if b4a
			pnl.SetLayoutAnimated(500, delta, pnl.Top, pnl.Width, pnl.Height)
			#else if b4i
			pnl.SetLayoutAnimated(500,1,delta,pnl.Top,pnl.Width,pnl.Height)
			#end if
		End If
	Else If Action = pnl.ACTION_UP Then
		If pnl.Left <> 0 Then	'if moved, so taps are ignored
			If LeftDirection Then
				#if b4a
				pnl.SetLayoutAnimated(500, -pnl.Left, pnl.Top, pnl.Width, pnl.Height)
				#else if b4i
				pnl.SetLayoutAnimated(500, 1,-pnl.Left, pnl.Top, pnl.Width, pnl.Height)
				#End If
			Else
				#if b4a
				pnl.SetLayoutAnimated(500, mBase.Width + 1dip, pnl.Top, pnl.Width, pnl.Height)
				#else if b4i
				pnl.SetLayoutAnimated(500, 1,mBase.Width + 1dip, pnl.Top, pnl.Width, pnl.Height)
				#End If
			End If
			pnl.SendToBack
			pnl.Left = 0
			Check_Next
			Dim a As Slide = lstSlides.Get(NextID)	'next image
			Dim NextPnl As Panel = a.pnl
			NextPnl.BringToFront
			NextPnl.Left = 0
			
			CurrentSlideID = NextID
			Update_Status
			Dim a As Slide = lstSlides.Get(NextID)
			#if B4a
			ToastMessageShow(a.Name, False)
			#end if
		End If
	End If
End Sub

Private Sub Check_Next
	'next slide image
	If NextID > lstSlides.Size -1 Then
		NextID = 0
	End If
	If NextID < 0 Then
		NextID = lstSlides.Size - 1
	End If
End Sub

'set the next image under the current, and hide others
Private Sub Hide_others
	For i = 0 To lstSlides.Size - 1
		Dim a As Slide
		a = lstSlides.Get(i)
		If i = NextID Or i = CurrentSlideID Then
		Else
			a.pnl.SendToBack
		End If
	Next
End Sub

Private Sub Update_Status
	If lstSlides.Size > 0 Then
		If lstSlides.Size <= 5 Then
			Dim cs As CSBuilder
			#if b4a
			cs.Initialize.Typeface(Typeface.FONTAWESOME)
			#else if b4i
			cs.Initialize.Font(Font.CreateFontAwesome(17))
			#End If
			
			For i = 0 To lstSlides.Size - 1
				If i = CurrentSlideID Then
					cs.Append(Chr(0xF111))
				Else
					cs.Append(Chr(0xF10C))
				End If
				cs.Append(" ")
			Next
			#if b4a
			lblStatus.Text = cs.PopAll
			#else if b4i
			cs.PopAll
			lblStatus.Text = cs.ToString
			#End If
		Else
			lblStatus.Text = (CurrentSlideID + 1) & " of " & lstSlides.Size
		End If
	Else
		lblStatus.Text = "No images loaded"
	End If
	lblStatus.BringToFront
End Sub


Sub FitCenterBitmapPanel(p As Panel,bmp As Bitmap,col As Int)
	Private cvs As Canvas
	
	cvs.Initialize(p)

	cvs.DrawColor(col)
	Dim rectDest As Rect
	Dim delta As Int

	If bmp.Width / bmp.Height > p.Width / p.Height Then
		delta = (p.Height - bmp.Height / bmp.Width * p.Width) / 2
		rectDest.Initialize(0, delta,p.Width, p.Height - delta)
	Else
		delta = (p.Width - bmp.Width / bmp.Height * p.Height) / 2
		rectDest.Initialize(delta, 0, p.Width - delta, p.Height)
	End If
	#if b4a
	cvs.DrawBitmap(bmp, Null, rectDest)
	#else if b4i
	cvs.DrawBitmap(bmp,rectDest)
	cvs.Refresh
	#End If
End Sub

#if b4i
private Sub SetBackgroundImage(p As Panel,bmp As Bitmap)
	Private cvs As Canvas
	Private rt As Rect
	
	rt.Initialize(0,0,p.Width,p.Height)
	cvs.Initialize(p)
	cvs.DrawBitmap(bmp,rt)
	cvs.Refresh
End Sub
#End If
